#version 430 core

layout(vertices = 16) out;

// Data of the camera
layout (std140, binding = 0) uniform CameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};

uniform vec2 window_size;				// Size of the window in pixels
uniform float triangle_size;			// Approximative size of one triangle in pixels when auto_tessellation is true
uniform bool auto_tessellation;			// Automatic / fixed tessellation factors
uniform float tessellation_factor;		// Tessellation factor when auto_tessellation is false

// Input variables
in VertexData
{
	vec3 position_ws;	// Position in world space
	vec3 tangent_ws;	// Tangent in world space
	vec3 bitangent_ws;	// Bitangent in world space
} inData[];

// Output variables
out VertexData
{
	vec3 position_ws;	// Position in world space
	vec3 tangent_ws;	// Tangent in world space
	vec3 bitangent_ws;	// Bitangent in world space
} outData[];

// Only to pass the data between invocations, not used in the evaluation shader
out vec2 position_ss[];		// Positions in screen space (i.e. after dividing by w and after multiplying by window size

void main()
{
	// Pass the position, tangent, and bitangent from inData to outData
	outData[gl_InvocationID].position_ws = inData[gl_InvocationID].position_ws;
	outData[gl_InvocationID].tangent_ws = inData[gl_InvocationID].tangent_ws;
	outData[gl_InvocationID].bitangent_ws = inData[gl_InvocationID].bitangent_ws;

	// Task 4: Compute the position of the control point in the screen space, and store it into shared 'position_ss' array
	//		Hint: Store the result into position_ss[gl_InvocationID], don't forget to call barrier()

	// Compute the tessellation factor
	if (gl_InvocationID == 0)
	{
		if (auto_tessellation)
		{
			// Task 4: Compute the tessellation factors
			//		Hint: function length
		}
		else
		{
			// Set all tessellation factors to 'tessellation_factor'.
			gl_TessLevelInner[0] = tessellation_factor;
			gl_TessLevelInner[1] = tessellation_factor;
			gl_TessLevelOuter[0] = tessellation_factor;
			gl_TessLevelOuter[1] = tessellation_factor;
			gl_TessLevelOuter[2] = tessellation_factor;
			gl_TessLevelOuter[3] = tessellation_factor;
		}
	}
}